@@ -17,7 +17,7 @@
 import dataclasses
 import json  # type: ignore
 import re
-from typing import Callable, Dict, List, Optional, Sequence, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
 import warnings
 
 from google.api_core import gapic_v1, path_template, rest_helpers, rest_streaming
@@ -274,7 +274,6 @@ class TemplatesServiceRestTransport(TemplatesServiceTransport):
                 request (~.templates.CreateJobFromTemplateRequest):
                     The request object. A request to create a Cloud Dataflow
                 job from a template.
-
                 retry (google.api_core.retry.Retry): Designation of what errors, if any,
                     should be retried.
                 timeout (float): The timeout for this request.
@@ -369,7 +368,6 @@ class TemplatesServiceRestTransport(TemplatesServiceTransport):
                 request (~.templates.GetTemplateRequest):
                     The request object. A request to retrieve a Cloud
                 Dataflow job template.
-
                 retry (google.api_core.retry.Retry): Designation of what errors, if any,
                     should be retried.
                 timeout (float): The timeout for this request.
