@@ -18,10 +18,12 @@ import os
 # try/except added for compatibility with python < 3.8
 try:
     from unittest import mock
-    from unittest.mock import AsyncMock
-except ImportError:
+    from unittest.mock import AsyncMock  # pragma: NO COVER
+except ImportError:  # pragma: NO COVER
     import mock
 
+from collections.abc import Iterable
+import json
 import math
 
 from google.api_core import gapic_v1, grpc_helpers, grpc_helpers_async, path_template
@@ -31,10 +33,14 @@ import google.auth
 from google.auth import credentials as ga_credentials
 from google.auth.exceptions import MutualTLSChannelError
 from google.oauth2 import service_account
+from google.protobuf import json_format
 import grpc
 from grpc.experimental import aio
+from proto.marshal.rules import wrappers
 from proto.marshal.rules.dates import DurationRule, TimestampRule
 import pytest
+from requests import PreparedRequest, Request, Response
+from requests.sessions import Session
 
 from google.cloud.dataflow_v1beta3.services.flex_templates_service import (
     FlexTemplatesServiceAsyncClient,
@@ -94,6 +100,7 @@ def test__get_default_mtls_endpoint():
     [
         (FlexTemplatesServiceClient, "grpc"),
         (FlexTemplatesServiceAsyncClient, "grpc_asyncio"),
+        (FlexTemplatesServiceClient, "rest"),
     ],
 )
 def test_flex_templates_service_client_from_service_account_info(
@@ -109,7 +116,11 @@ def test_flex_templates_service_client_from_service_account_info(
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == ("dataflow.googleapis.com:443")
+        assert client.transport._host == (
+            "dataflow.googleapis.com:443"
+            if transport_name in ["grpc", "grpc_asyncio"]
+            else "https://dataflow.googleapis.com"
+        )
 
 
 @pytest.mark.parametrize(
@@ -117,6 +128,7 @@ def test_flex_templates_service_client_from_service_account_info(
     [
         (transports.FlexTemplatesServiceGrpcTransport, "grpc"),
         (transports.FlexTemplatesServiceGrpcAsyncIOTransport, "grpc_asyncio"),
+        (transports.FlexTemplatesServiceRestTransport, "rest"),
     ],
 )
 def test_flex_templates_service_client_service_account_always_use_jwt(
@@ -142,6 +154,7 @@ def test_flex_templates_service_client_service_account_always_use_jwt(
     [
         (FlexTemplatesServiceClient, "grpc"),
         (FlexTemplatesServiceAsyncClient, "grpc_asyncio"),
+        (FlexTemplatesServiceClient, "rest"),
     ],
 )
 def test_flex_templates_service_client_from_service_account_file(
@@ -164,13 +177,18 @@ def test_flex_templates_service_client_from_service_account_file(
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == ("dataflow.googleapis.com:443")
+        assert client.transport._host == (
+            "dataflow.googleapis.com:443"
+            if transport_name in ["grpc", "grpc_asyncio"]
+            else "https://dataflow.googleapis.com"
+        )
 
 
 def test_flex_templates_service_client_get_transport_class():
     transport = FlexTemplatesServiceClient.get_transport_class()
     available_transports = [
         transports.FlexTemplatesServiceGrpcTransport,
+        transports.FlexTemplatesServiceRestTransport,
     ]
     assert transport in available_transports
 
@@ -191,6 +209,11 @@ def test_flex_templates_service_client_get_transport_class():
             transports.FlexTemplatesServiceGrpcAsyncIOTransport,
             "grpc_asyncio",
         ),
+        (
+            FlexTemplatesServiceClient,
+            transports.FlexTemplatesServiceRestTransport,
+            "rest",
+        ),
     ],
 )
 @mock.patch.object(
@@ -346,6 +369,18 @@ def test_flex_templates_service_client_client_options(
             "grpc_asyncio",
             "false",
         ),
+        (
+            FlexTemplatesServiceClient,
+            transports.FlexTemplatesServiceRestTransport,
+            "rest",
+            "true",
+        ),
+        (
+            FlexTemplatesServiceClient,
+            transports.FlexTemplatesServiceRestTransport,
+            "rest",
+            "false",
+        ),
     ],
 )
 @mock.patch.object(
@@ -549,6 +584,11 @@ def test_flex_templates_service_client_get_mtls_endpoint_and_cert_source(client_
             transports.FlexTemplatesServiceGrpcAsyncIOTransport,
             "grpc_asyncio",
         ),
+        (
+            FlexTemplatesServiceClient,
+            transports.FlexTemplatesServiceRestTransport,
+            "rest",
+        ),
     ],
 )
 def test_flex_templates_service_client_client_options_scopes(
@@ -589,6 +629,12 @@ def test_flex_templates_service_client_client_options_scopes(
             "grpc_asyncio",
             grpc_helpers_async,
         ),
+        (
+            FlexTemplatesServiceClient,
+            transports.FlexTemplatesServiceRestTransport,
+            "rest",
+            None,
+        ),
     ],
 )
 def test_flex_templates_service_client_client_options_credentials_file(
@@ -862,6 +908,129 @@ async def test_launch_flex_template_field_headers_async():
     ) in kw["metadata"]
 
 
+@pytest.mark.parametrize(
+    "request_type",
+    [
+        templates.LaunchFlexTemplateRequest,
+        dict,
+    ],
+)
+def test_launch_flex_template_rest(request_type):
+    client = FlexTemplatesServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {"project_id": "sample1", "location": "sample2"}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = templates.LaunchFlexTemplateResponse()
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        pb_return_value = templates.LaunchFlexTemplateResponse.pb(return_value)
+        json_return_value = json_format.MessageToJson(pb_return_value)
+
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+        response = client.launch_flex_template(request)
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, templates.LaunchFlexTemplateResponse)
+
+
+@pytest.mark.parametrize("null_interceptor", [True, False])
+def test_launch_flex_template_rest_interceptors(null_interceptor):
+    transport = transports.FlexTemplatesServiceRestTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+        interceptor=None
+        if null_interceptor
+        else transports.FlexTemplatesServiceRestInterceptor(),
+    )
+    client = FlexTemplatesServiceClient(transport=transport)
+    with mock.patch.object(
+        type(client.transport._session), "request"
+    ) as req, mock.patch.object(
+        path_template, "transcode"
+    ) as transcode, mock.patch.object(
+        transports.FlexTemplatesServiceRestInterceptor, "post_launch_flex_template"
+    ) as post, mock.patch.object(
+        transports.FlexTemplatesServiceRestInterceptor, "pre_launch_flex_template"
+    ) as pre:
+        pre.assert_not_called()
+        post.assert_not_called()
+        pb_message = templates.LaunchFlexTemplateRequest.pb(
+            templates.LaunchFlexTemplateRequest()
+        )
+        transcode.return_value = {
+            "method": "post",
+            "uri": "my_uri",
+            "body": pb_message,
+            "query_params": pb_message,
+        }
+
+        req.return_value = Response()
+        req.return_value.status_code = 200
+        req.return_value.request = PreparedRequest()
+        req.return_value._content = templates.LaunchFlexTemplateResponse.to_json(
+            templates.LaunchFlexTemplateResponse()
+        )
+
+        request = templates.LaunchFlexTemplateRequest()
+        metadata = [
+            ("key", "val"),
+            ("cephalopod", "squid"),
+        ]
+        pre.return_value = request, metadata
+        post.return_value = templates.LaunchFlexTemplateResponse()
+
+        client.launch_flex_template(
+            request,
+            metadata=[
+                ("key", "val"),
+                ("cephalopod", "squid"),
+            ],
+        )
+
+        pre.assert_called_once()
+        post.assert_called_once()
+
+
+def test_launch_flex_template_rest_bad_request(
+    transport: str = "rest", request_type=templates.LaunchFlexTemplateRequest
+):
+    client = FlexTemplatesServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {"project_id": "sample1", "location": "sample2"}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a BadRequest error.
+    with mock.patch.object(Session, "request") as req, pytest.raises(
+        core_exceptions.BadRequest
+    ):
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 400
+        response_value.request = Request()
+        req.return_value = response_value
+        client.launch_flex_template(request)
+
+
+def test_launch_flex_template_rest_error():
+    client = FlexTemplatesServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+
+
 def test_credentials_transport_error():
     # It is an error to provide credentials and a transport instance.
     transport = transports.FlexTemplatesServiceGrpcTransport(
@@ -943,6 +1112,7 @@ def test_transport_get_channel():
     [
         transports.FlexTemplatesServiceGrpcTransport,
         transports.FlexTemplatesServiceGrpcAsyncIOTransport,
+        transports.FlexTemplatesServiceRestTransport,
     ],
 )
 def test_transport_adc(transport_class):
@@ -957,6 +1127,7 @@ def test_transport_adc(transport_class):
     "transport_name",
     [
         "grpc",
+        "rest",
     ],
 )
 def test_transport_kind(transport_name):
@@ -1099,6 +1270,7 @@ def test_flex_templates_service_transport_auth_adc(transport_class):
     [
         transports.FlexTemplatesServiceGrpcTransport,
         transports.FlexTemplatesServiceGrpcAsyncIOTransport,
+        transports.FlexTemplatesServiceRestTransport,
     ],
 )
 def test_flex_templates_service_transport_auth_gdch_credentials(transport_class):
@@ -1203,11 +1375,23 @@ def test_flex_templates_service_grpc_transport_client_cert_source_for_mtls(
             )
 
 
+def test_flex_templates_service_http_transport_client_cert_source_for_mtls():
+    cred = ga_credentials.AnonymousCredentials()
+    with mock.patch(
+        "google.auth.transport.requests.AuthorizedSession.configure_mtls_channel"
+    ) as mock_configure_mtls_channel:
+        transports.FlexTemplatesServiceRestTransport(
+            credentials=cred, client_cert_source_for_mtls=client_cert_source_callback
+        )
+        mock_configure_mtls_channel.assert_called_once_with(client_cert_source_callback)
+
+
 @pytest.mark.parametrize(
     "transport_name",
     [
         "grpc",
         "grpc_asyncio",
+        "rest",
     ],
 )
 def test_flex_templates_service_host_no_port(transport_name):
@@ -1218,7 +1402,11 @@ def test_flex_templates_service_host_no_port(transport_name):
         ),
         transport=transport_name,
     )
-    assert client.transport._host == ("dataflow.googleapis.com:443")
+    assert client.transport._host == (
+        "dataflow.googleapis.com:443"
+        if transport_name in ["grpc", "grpc_asyncio"]
+        else "https://dataflow.googleapis.com"
+    )
 
 
 @pytest.mark.parametrize(
@@ -1226,6 +1414,7 @@ def test_flex_templates_service_host_no_port(transport_name):
     [
         "grpc",
         "grpc_asyncio",
+        "rest",
     ],
 )
 def test_flex_templates_service_host_with_port(transport_name):
@@ -1236,7 +1425,33 @@ def test_flex_templates_service_host_with_port(transport_name):
         ),
         transport=transport_name,
     )
-    assert client.transport._host == ("dataflow.googleapis.com:8000")
+    assert client.transport._host == (
+        "dataflow.googleapis.com:8000"
+        if transport_name in ["grpc", "grpc_asyncio"]
+        else "https://dataflow.googleapis.com:8000"
+    )
+
+
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "rest",
+    ],
+)
+def test_flex_templates_service_client_transport_session_collision(transport_name):
+    creds1 = ga_credentials.AnonymousCredentials()
+    creds2 = ga_credentials.AnonymousCredentials()
+    client1 = FlexTemplatesServiceClient(
+        credentials=creds1,
+        transport=transport_name,
+    )
+    client2 = FlexTemplatesServiceClient(
+        credentials=creds2,
+        transport=transport_name,
+    )
+    session1 = client1.transport.launch_flex_template._session
+    session2 = client2.transport.launch_flex_template._session
+    assert session1 != session2
 
 
 def test_flex_templates_service_grpc_transport_channel():
@@ -1507,6 +1722,7 @@ async def test_transport_close_async():
 
 def test_transport_close():
     transports = {
+        "rest": "_session",
         "grpc": "_grpc_channel",
     }
 
@@ -1524,6 +1740,7 @@ def test_transport_close():
 
 def test_client_ctx():
     transports = [
+        "rest",
         "grpc",
     ]
     for transport in transports:
