@@ -18,10 +18,12 @@ import os
 # try/except added for compatibility with python < 3.8
 try:
     from unittest import mock
-    from unittest.mock import AsyncMock
-except ImportError:
+    from unittest.mock import AsyncMock  # pragma: NO COVER
+except ImportError:  # pragma: NO COVER
     import mock
 
+from collections.abc import Iterable
+import json
 import math
 
 from google.api_core import gapic_v1, grpc_helpers, grpc_helpers_async, path_template
@@ -32,11 +34,15 @@ from google.auth import credentials as ga_credentials
 from google.auth.exceptions import MutualTLSChannelError
 from google.oauth2 import service_account
 from google.protobuf import duration_pb2  # type: ignore
+from google.protobuf import json_format
 from google.protobuf import timestamp_pb2  # type: ignore
 import grpc
 from grpc.experimental import aio
+from proto.marshal.rules import wrappers
 from proto.marshal.rules.dates import DurationRule, TimestampRule
 import pytest
+from requests import PreparedRequest, Request, Response
+from requests.sessions import Session
 
 from google.cloud.dataflow_v1beta3.services.snapshots_v1_beta3 import (
     SnapshotsV1Beta3AsyncClient,
@@ -96,6 +102,7 @@ def test__get_default_mtls_endpoint():
     [
         (SnapshotsV1Beta3Client, "grpc"),
         (SnapshotsV1Beta3AsyncClient, "grpc_asyncio"),
+        (SnapshotsV1Beta3Client, "rest"),
     ],
 )
 def test_snapshots_v1_beta3_client_from_service_account_info(
@@ -111,7 +118,11 @@ def test_snapshots_v1_beta3_client_from_service_account_info(
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == ("dataflow.googleapis.com:443")
+        assert client.transport._host == (
+            "dataflow.googleapis.com:443"
+            if transport_name in ["grpc", "grpc_asyncio"]
+            else "https://dataflow.googleapis.com"
+        )
 
 
 @pytest.mark.parametrize(
@@ -119,6 +130,7 @@ def test_snapshots_v1_beta3_client_from_service_account_info(
     [
         (transports.SnapshotsV1Beta3GrpcTransport, "grpc"),
         (transports.SnapshotsV1Beta3GrpcAsyncIOTransport, "grpc_asyncio"),
+        (transports.SnapshotsV1Beta3RestTransport, "rest"),
     ],
 )
 def test_snapshots_v1_beta3_client_service_account_always_use_jwt(
@@ -144,6 +156,7 @@ def test_snapshots_v1_beta3_client_service_account_always_use_jwt(
     [
         (SnapshotsV1Beta3Client, "grpc"),
         (SnapshotsV1Beta3AsyncClient, "grpc_asyncio"),
+        (SnapshotsV1Beta3Client, "rest"),
     ],
 )
 def test_snapshots_v1_beta3_client_from_service_account_file(
@@ -166,13 +179,18 @@ def test_snapshots_v1_beta3_client_from_service_account_file(
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == ("dataflow.googleapis.com:443")
+        assert client.transport._host == (
+            "dataflow.googleapis.com:443"
+            if transport_name in ["grpc", "grpc_asyncio"]
+            else "https://dataflow.googleapis.com"
+        )
 
 
 def test_snapshots_v1_beta3_client_get_transport_class():
     transport = SnapshotsV1Beta3Client.get_transport_class()
     available_transports = [
         transports.SnapshotsV1Beta3GrpcTransport,
+        transports.SnapshotsV1Beta3RestTransport,
     ]
     assert transport in available_transports
 
@@ -189,6 +207,7 @@ def test_snapshots_v1_beta3_client_get_transport_class():
             transports.SnapshotsV1Beta3GrpcAsyncIOTransport,
             "grpc_asyncio",
         ),
+        (SnapshotsV1Beta3Client, transports.SnapshotsV1Beta3RestTransport, "rest"),
     ],
 )
 @mock.patch.object(
@@ -344,6 +363,18 @@ def test_snapshots_v1_beta3_client_client_options(
             "grpc_asyncio",
             "false",
         ),
+        (
+            SnapshotsV1Beta3Client,
+            transports.SnapshotsV1Beta3RestTransport,
+            "rest",
+            "true",
+        ),
+        (
+            SnapshotsV1Beta3Client,
+            transports.SnapshotsV1Beta3RestTransport,
+            "rest",
+            "false",
+        ),
     ],
 )
 @mock.patch.object(
@@ -543,6 +574,7 @@ def test_snapshots_v1_beta3_client_get_mtls_endpoint_and_cert_source(client_clas
             transports.SnapshotsV1Beta3GrpcAsyncIOTransport,
             "grpc_asyncio",
         ),
+        (SnapshotsV1Beta3Client, transports.SnapshotsV1Beta3RestTransport, "rest"),
     ],
 )
 def test_snapshots_v1_beta3_client_client_options_scopes(
@@ -583,6 +615,12 @@ def test_snapshots_v1_beta3_client_client_options_scopes(
             "grpc_asyncio",
             grpc_helpers_async,
         ),
+        (
+            SnapshotsV1Beta3Client,
+            transports.SnapshotsV1Beta3RestTransport,
+            "rest",
+            None,
+        ),
     ],
 )
 def test_snapshots_v1_beta3_client_client_options_credentials_file(
@@ -1172,6 +1210,400 @@ async def test_list_snapshots_field_headers_async():
     ) in kw["metadata"]
 
 
+@pytest.mark.parametrize(
+    "request_type",
+    [
+        snapshots.GetSnapshotRequest,
+        dict,
+    ],
+)
+def test_get_snapshot_rest(request_type):
+    client = SnapshotsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {
+        "project_id": "sample1",
+        "location": "sample2",
+        "snapshot_id": "sample3",
+    }
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = snapshots.Snapshot(
+            id="id_value",
+            project_id="project_id_value",
+            source_job_id="source_job_id_value",
+            state=snapshots.SnapshotState.PENDING,
+            description="description_value",
+            disk_size_bytes=1611,
+            region="region_value",
+        )
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        pb_return_value = snapshots.Snapshot.pb(return_value)
+        json_return_value = json_format.MessageToJson(pb_return_value)
+
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+        response = client.get_snapshot(request)
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, snapshots.Snapshot)
+    assert response.id == "id_value"
+    assert response.project_id == "project_id_value"
+    assert response.source_job_id == "source_job_id_value"
+    assert response.state == snapshots.SnapshotState.PENDING
+    assert response.description == "description_value"
+    assert response.disk_size_bytes == 1611
+    assert response.region == "region_value"
+
+
+@pytest.mark.parametrize("null_interceptor", [True, False])
+def test_get_snapshot_rest_interceptors(null_interceptor):
+    transport = transports.SnapshotsV1Beta3RestTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+        interceptor=None
+        if null_interceptor
+        else transports.SnapshotsV1Beta3RestInterceptor(),
+    )
+    client = SnapshotsV1Beta3Client(transport=transport)
+    with mock.patch.object(
+        type(client.transport._session), "request"
+    ) as req, mock.patch.object(
+        path_template, "transcode"
+    ) as transcode, mock.patch.object(
+        transports.SnapshotsV1Beta3RestInterceptor, "post_get_snapshot"
+    ) as post, mock.patch.object(
+        transports.SnapshotsV1Beta3RestInterceptor, "pre_get_snapshot"
+    ) as pre:
+        pre.assert_not_called()
+        post.assert_not_called()
+        pb_message = snapshots.GetSnapshotRequest.pb(snapshots.GetSnapshotRequest())
+        transcode.return_value = {
+            "method": "post",
+            "uri": "my_uri",
+            "body": pb_message,
+            "query_params": pb_message,
+        }
+
+        req.return_value = Response()
+        req.return_value.status_code = 200
+        req.return_value.request = PreparedRequest()
+        req.return_value._content = snapshots.Snapshot.to_json(snapshots.Snapshot())
+
+        request = snapshots.GetSnapshotRequest()
+        metadata = [
+            ("key", "val"),
+            ("cephalopod", "squid"),
+        ]
+        pre.return_value = request, metadata
+        post.return_value = snapshots.Snapshot()
+
+        client.get_snapshot(
+            request,
+            metadata=[
+                ("key", "val"),
+                ("cephalopod", "squid"),
+            ],
+        )
+
+        pre.assert_called_once()
+        post.assert_called_once()
+
+
+def test_get_snapshot_rest_bad_request(
+    transport: str = "rest", request_type=snapshots.GetSnapshotRequest
+):
+    client = SnapshotsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {
+        "project_id": "sample1",
+        "location": "sample2",
+        "snapshot_id": "sample3",
+    }
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a BadRequest error.
+    with mock.patch.object(Session, "request") as req, pytest.raises(
+        core_exceptions.BadRequest
+    ):
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 400
+        response_value.request = Request()
+        req.return_value = response_value
+        client.get_snapshot(request)
+
+
+def test_get_snapshot_rest_error():
+    client = SnapshotsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+
+
+@pytest.mark.parametrize(
+    "request_type",
+    [
+        snapshots.DeleteSnapshotRequest,
+        dict,
+    ],
+)
+def test_delete_snapshot_rest(request_type):
+    client = SnapshotsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {
+        "project_id": "sample1",
+        "location": "sample2",
+        "snapshot_id": "sample3",
+    }
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = snapshots.DeleteSnapshotResponse()
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        pb_return_value = snapshots.DeleteSnapshotResponse.pb(return_value)
+        json_return_value = json_format.MessageToJson(pb_return_value)
+
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+        response = client.delete_snapshot(request)
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, snapshots.DeleteSnapshotResponse)
+
+
+@pytest.mark.parametrize("null_interceptor", [True, False])
+def test_delete_snapshot_rest_interceptors(null_interceptor):
+    transport = transports.SnapshotsV1Beta3RestTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+        interceptor=None
+        if null_interceptor
+        else transports.SnapshotsV1Beta3RestInterceptor(),
+    )
+    client = SnapshotsV1Beta3Client(transport=transport)
+    with mock.patch.object(
+        type(client.transport._session), "request"
+    ) as req, mock.patch.object(
+        path_template, "transcode"
+    ) as transcode, mock.patch.object(
+        transports.SnapshotsV1Beta3RestInterceptor, "post_delete_snapshot"
+    ) as post, mock.patch.object(
+        transports.SnapshotsV1Beta3RestInterceptor, "pre_delete_snapshot"
+    ) as pre:
+        pre.assert_not_called()
+        post.assert_not_called()
+        pb_message = snapshots.DeleteSnapshotRequest.pb(
+            snapshots.DeleteSnapshotRequest()
+        )
+        transcode.return_value = {
+            "method": "post",
+            "uri": "my_uri",
+            "body": pb_message,
+            "query_params": pb_message,
+        }
+
+        req.return_value = Response()
+        req.return_value.status_code = 200
+        req.return_value.request = PreparedRequest()
+        req.return_value._content = snapshots.DeleteSnapshotResponse.to_json(
+            snapshots.DeleteSnapshotResponse()
+        )
+
+        request = snapshots.DeleteSnapshotRequest()
+        metadata = [
+            ("key", "val"),
+            ("cephalopod", "squid"),
+        ]
+        pre.return_value = request, metadata
+        post.return_value = snapshots.DeleteSnapshotResponse()
+
+        client.delete_snapshot(
+            request,
+            metadata=[
+                ("key", "val"),
+                ("cephalopod", "squid"),
+            ],
+        )
+
+        pre.assert_called_once()
+        post.assert_called_once()
+
+
+def test_delete_snapshot_rest_bad_request(
+    transport: str = "rest", request_type=snapshots.DeleteSnapshotRequest
+):
+    client = SnapshotsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {
+        "project_id": "sample1",
+        "location": "sample2",
+        "snapshot_id": "sample3",
+    }
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a BadRequest error.
+    with mock.patch.object(Session, "request") as req, pytest.raises(
+        core_exceptions.BadRequest
+    ):
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 400
+        response_value.request = Request()
+        req.return_value = response_value
+        client.delete_snapshot(request)
+
+
+def test_delete_snapshot_rest_error():
+    client = SnapshotsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+
+
+@pytest.mark.parametrize(
+    "request_type",
+    [
+        snapshots.ListSnapshotsRequest,
+        dict,
+    ],
+)
+def test_list_snapshots_rest(request_type):
+    client = SnapshotsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {"project_id": "sample1", "location": "sample2", "job_id": "sample3"}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = snapshots.ListSnapshotsResponse()
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        pb_return_value = snapshots.ListSnapshotsResponse.pb(return_value)
+        json_return_value = json_format.MessageToJson(pb_return_value)
+
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+        response = client.list_snapshots(request)
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, snapshots.ListSnapshotsResponse)
+
+
+@pytest.mark.parametrize("null_interceptor", [True, False])
+def test_list_snapshots_rest_interceptors(null_interceptor):
+    transport = transports.SnapshotsV1Beta3RestTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+        interceptor=None
+        if null_interceptor
+        else transports.SnapshotsV1Beta3RestInterceptor(),
+    )
+    client = SnapshotsV1Beta3Client(transport=transport)
+    with mock.patch.object(
+        type(client.transport._session), "request"
+    ) as req, mock.patch.object(
+        path_template, "transcode"
+    ) as transcode, mock.patch.object(
+        transports.SnapshotsV1Beta3RestInterceptor, "post_list_snapshots"
+    ) as post, mock.patch.object(
+        transports.SnapshotsV1Beta3RestInterceptor, "pre_list_snapshots"
+    ) as pre:
+        pre.assert_not_called()
+        post.assert_not_called()
+        pb_message = snapshots.ListSnapshotsRequest.pb(snapshots.ListSnapshotsRequest())
+        transcode.return_value = {
+            "method": "post",
+            "uri": "my_uri",
+            "body": pb_message,
+            "query_params": pb_message,
+        }
+
+        req.return_value = Response()
+        req.return_value.status_code = 200
+        req.return_value.request = PreparedRequest()
+        req.return_value._content = snapshots.ListSnapshotsResponse.to_json(
+            snapshots.ListSnapshotsResponse()
+        )
+
+        request = snapshots.ListSnapshotsRequest()
+        metadata = [
+            ("key", "val"),
+            ("cephalopod", "squid"),
+        ]
+        pre.return_value = request, metadata
+        post.return_value = snapshots.ListSnapshotsResponse()
+
+        client.list_snapshots(
+            request,
+            metadata=[
+                ("key", "val"),
+                ("cephalopod", "squid"),
+            ],
+        )
+
+        pre.assert_called_once()
+        post.assert_called_once()
+
+
+def test_list_snapshots_rest_bad_request(
+    transport: str = "rest", request_type=snapshots.ListSnapshotsRequest
+):
+    client = SnapshotsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {"project_id": "sample1", "location": "sample2", "job_id": "sample3"}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a BadRequest error.
+    with mock.patch.object(Session, "request") as req, pytest.raises(
+        core_exceptions.BadRequest
+    ):
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 400
+        response_value.request = Request()
+        req.return_value = response_value
+        client.list_snapshots(request)
+
+
+def test_list_snapshots_rest_error():
+    client = SnapshotsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+
+
 def test_credentials_transport_error():
     # It is an error to provide credentials and a transport instance.
     transport = transports.SnapshotsV1Beta3GrpcTransport(
@@ -1253,6 +1685,7 @@ def test_transport_get_channel():
     [
         transports.SnapshotsV1Beta3GrpcTransport,
         transports.SnapshotsV1Beta3GrpcAsyncIOTransport,
+        transports.SnapshotsV1Beta3RestTransport,
     ],
 )
 def test_transport_adc(transport_class):
@@ -1267,6 +1700,7 @@ def test_transport_adc(transport_class):
     "transport_name",
     [
         "grpc",
+        "rest",
     ],
 )
 def test_transport_kind(transport_name):
@@ -1413,6 +1847,7 @@ def test_snapshots_v1_beta3_transport_auth_adc(transport_class):
     [
         transports.SnapshotsV1Beta3GrpcTransport,
         transports.SnapshotsV1Beta3GrpcAsyncIOTransport,
+        transports.SnapshotsV1Beta3RestTransport,
     ],
 )
 def test_snapshots_v1_beta3_transport_auth_gdch_credentials(transport_class):
@@ -1515,11 +1950,23 @@ def test_snapshots_v1_beta3_grpc_transport_client_cert_source_for_mtls(transport
             )
 
 
+def test_snapshots_v1_beta3_http_transport_client_cert_source_for_mtls():
+    cred = ga_credentials.AnonymousCredentials()
+    with mock.patch(
+        "google.auth.transport.requests.AuthorizedSession.configure_mtls_channel"
+    ) as mock_configure_mtls_channel:
+        transports.SnapshotsV1Beta3RestTransport(
+            credentials=cred, client_cert_source_for_mtls=client_cert_source_callback
+        )
+        mock_configure_mtls_channel.assert_called_once_with(client_cert_source_callback)
+
+
 @pytest.mark.parametrize(
     "transport_name",
     [
         "grpc",
         "grpc_asyncio",
+        "rest",
     ],
 )
 def test_snapshots_v1_beta3_host_no_port(transport_name):
@@ -1530,7 +1977,11 @@ def test_snapshots_v1_beta3_host_no_port(transport_name):
         ),
         transport=transport_name,
     )
-    assert client.transport._host == ("dataflow.googleapis.com:443")
+    assert client.transport._host == (
+        "dataflow.googleapis.com:443"
+        if transport_name in ["grpc", "grpc_asyncio"]
+        else "https://dataflow.googleapis.com"
+    )
 
 
 @pytest.mark.parametrize(
@@ -1538,6 +1989,7 @@ def test_snapshots_v1_beta3_host_no_port(transport_name):
     [
         "grpc",
         "grpc_asyncio",
+        "rest",
     ],
 )
 def test_snapshots_v1_beta3_host_with_port(transport_name):
@@ -1548,7 +2000,39 @@ def test_snapshots_v1_beta3_host_with_port(transport_name):
         ),
         transport=transport_name,
     )
-    assert client.transport._host == ("dataflow.googleapis.com:8000")
+    assert client.transport._host == (
+        "dataflow.googleapis.com:8000"
+        if transport_name in ["grpc", "grpc_asyncio"]
+        else "https://dataflow.googleapis.com:8000"
+    )
+
+
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "rest",
+    ],
+)
+def test_snapshots_v1_beta3_client_transport_session_collision(transport_name):
+    creds1 = ga_credentials.AnonymousCredentials()
+    creds2 = ga_credentials.AnonymousCredentials()
+    client1 = SnapshotsV1Beta3Client(
+        credentials=creds1,
+        transport=transport_name,
+    )
+    client2 = SnapshotsV1Beta3Client(
+        credentials=creds2,
+        transport=transport_name,
+    )
+    session1 = client1.transport.get_snapshot._session
+    session2 = client2.transport.get_snapshot._session
+    assert session1 != session2
+    session1 = client1.transport.delete_snapshot._session
+    session2 = client2.transport.delete_snapshot._session
+    assert session1 != session2
+    session1 = client1.transport.list_snapshots._session
+    session2 = client2.transport.list_snapshots._session
+    assert session1 != session2
 
 
 def test_snapshots_v1_beta3_grpc_transport_channel():
@@ -1819,6 +2303,7 @@ async def test_transport_close_async():
 
 def test_transport_close():
     transports = {
+        "rest": "_session",
         "grpc": "_grpc_channel",
     }
 
@@ -1836,6 +2321,7 @@ def test_transport_close():
 
 def test_client_ctx():
     transports = [
+        "rest",
         "grpc",
     ]
     for transport in transports:
